const char init_lua[] =
"\n"
"-- init.lua -- internal file\n"
"\n"
"local ffi = require('ffi')\n"
"ffi.cdef[[\n"
"struct type_info;\n"
"struct method_info;\n"
"\n"
"enum ctype {\n"
"    CTYPE_VOID = 0,\n"
"    CTYPE_INT,\n"
"    CTYPE_CONST_CHAR_PTR\n"
"};\n"
"\n"
"struct type_info {\n"
"    const char *name;\n"
"    const struct type_info *parent;\n"
"    const struct method_info *methods;\n"
"};\n"
"\n"
"enum { METHOD_ARG_MAX = 8 };\n"
"\n"
"struct method_info {\n"
"    const struct type_info *owner;\n"
"    const char *name;\n"
"    enum ctype rtype;\n"
"    enum ctype atype[METHOD_ARG_MAX];\n"
"    int nargs;\n"
"    bool isconst;\n"
"\n"
"    union {\n"
"        /* Add extra space to get proper struct size in C */\n"
"        void *_spacer[2];\n"
"    };\n"
"};\n"
"\n"
"double\n"
"tarantool_uptime(void);\n"
"typedef int32_t pid_t;\n"
"pid_t getpid(void);\n"
"void\n"
"tarantool_exit(int);\n"
"]]\n"
"\n"
"local fio = require(\"fio\")\n"
"\n"
"dostring = function(s, ...)\n"
"    local chunk, message = loadstring(s)\n"
"    if chunk == nil then\n"
"        error(message, 2)\n"
"    end\n"
"    return chunk(...)\n"
"end\n"
"\n"
"local fiber = require(\"fiber\")\n"
"os.exit = function(code)\n"
"    code = (type(code) == 'number') and code or 0\n"
"    ffi.C.tarantool_exit(code)\n"
"    -- Make sure we yield even if the code after\n"
"    -- os.exit() never yields. After on_shutdown\n"
"    -- fiber completes, we will never wake up again.\n"
"    while true do fiber.yield() end\n"
"end\n"
"\n"
"local function uptime()\n"
"    return tonumber(ffi.C.tarantool_uptime());\n"
"end\n"
"\n"
"local function pid()\n"
"    return tonumber(ffi.C.getpid())\n"
"end\n"
"\n"
"local soext = (jit.os == \"OSX\" and \"dylib\" or \"so\")\n"
"\n"
"local function mksymname(name)\n"
"    local mark = string.find(name, \"-\")\n"
"    if mark then name = string.sub(name, mark + 1) end\n"
"    return \"luaopen_\" .. string.gsub(name, \"%.\", \"_\")\n"
"end\n"
"\n"
"local function load_lib(file, name)\n"
"    return package.loadlib(file, mksymname(name))\n"
"end\n"
"\n"
"local function load_lua(file)\n"
"    return loadfile(file)\n"
"end\n"
"\n"
"local function search_cwd_lib(name)\n"
"    local path = \"./?.\"..soext\n"
"    return package.searchpath(name, path)\n"
"end\n"
"\n"
"local function search_cwd_lua(name)\n"
"    local path = \"./?.lua;./?/init.lua\"\n"
"    return package.searchpath(name, path)\n"
"end\n"
"\n"
"local function traverse_rocks(name, pathes_search)\n"
"    local cwd = fio.cwd()\n"
"    local index = string.len(cwd) + 1\n"
"    local strerr = \"\"\n"
"    while index ~= nil do\n"
"        cwd = string.sub(cwd, 1, index - 1)\n"
"        for i, path in ipairs(pathes_search) do\n"
"            local file, err = package.searchpath(name, cwd .. path)\n"
"            if err == nil then\n"
"                return file\n"
"            end\n"
"            strerr = strerr .. err\n"
"        end\n"
"        index = string.find(cwd, \"/[^/]*$\")\n"
"    end\n"
"    return nil, strerr\n"
"end\n"
"\n"
"local function search_rocks_lua(name)\n"
"    local pathes_search = {\n"
"        \"/.rocks/share/tarantool/?.lua;\",\n"
"        \"/.rocks/share/tarantool/?/init.lua;\",\n"
"    }\n"
"    return traverse_rocks(name, pathes_search)\n"
"end\n"
"\n"
"local function search_rocks_lib(name)\n"
"    local pathes_search = {\n"
"        \"/.rocks/lib/tarantool/?.\"..soext\n"
"    }\n"
"    return traverse_rocks(name, pathes_search)\n"
"end\n"
"\n"
"local function cwd_loader_func(lib)\n"
"    local search_cwd = lib and search_cwd_lib or search_cwd_lua\n"
"    local load_func = lib and load_lib or load_lua\n"
"    return function(name)\n"
"        if not name then\n"
"            return \"empty name of module\"\n"
"        end\n"
"        local file, err = search_cwd(name)\n"
"        if not file then\n"
"            return err\n"
"        end\n"
"        local loaded, err = load_func(file, name)\n"
"        if err == nil then\n"
"            return loaded\n"
"        else\n"
"            return err\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"local function rocks_loader_func(lib)\n"
"    local search_rocks = lib and search_rocks_lib or search_rocks_lua\n"
"    local load_func = lib and load_lib or load_lua\n"
"    return function (name)\n"
"        if not name then\n"
"            return \"empty name of module\"\n"
"        end\n"
"        local file, err = search_rocks(name)\n"
"        if not file then\n"
"            return err\n"
"        end\n"
"        local loaded, err = load_func(file, name)\n"
"        if err == nil then\n"
"            return loaded\n"
"        else\n"
"            return err\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"local function search_path_func(cpath)\n"
"    return function(name)\n"
"        return package.searchpath(name, cpath and package.cpath or package.path)\n"
"    end\n"
"end\n"
"\n"
"local function search(name)\n"
"    if not name then\n"
"        return \"empty name of module\"\n"
"    end\n"
"    local searchers = {\n"
"        search_cwd_lua, search_cwd_lib,\n"
"        search_rocks_lua, search_rocks_lib,\n"
"        search_path_func(false), search_path_func(true)\n"
"    }\n"
"    for _, searcher in ipairs(searchers) do\n"
"        local file = searcher(name)\n"
"        if file ~= nil then\n"
"            return file\n"
"        end\n"
"    end\n"
"    return nil\n"
"end\n"
"\n"
"-- loader_preload 1\n"
"table.insert(package.loaders, 2, cwd_loader_func(false))\n"
"table.insert(package.loaders, 3, cwd_loader_func(true))\n"
"table.insert(package.loaders, 4, rocks_loader_func(false))\n"
"table.insert(package.loaders, 5, rocks_loader_func(true))\n"
"-- package.path   6\n"
"-- package.cpath  7\n"
"-- croot          8\n"
"\n"
"package.search = search\n"
"\n"
"return {\n"
"    uptime = uptime;\n"
"    pid = pid;\n"
"}\n"
""
;
